inlets = 2;
outlets = 2;

var curr_priority = 0;	// Priority values:
						// 0 = Last note priority
						// 1 = High note priority
						// 2 = Low note priority

var last_velo = 0;		// the last velocity received
var last_note = 0;		// the late note received

var note_array = new Array();
var last_array = new Array();

// initialize the arrays
for (var i=0; i<128; i++) {
	note_array[i] = 0;
}

var play_note = -1;		// the currently (or last) played note

// note playback queueing system
const QUEUEMS = 2;		// 2 ms queue time
var t = new Task(queueplay, this);
var isQueing = 0;
var queueNote = -1;
var queueVel  = -1;


//	Message handlers
//	================

function info() {
	var i;
	
	post("Play note: ", play_note, '\n');
	
	post("Note array: \n");
	for (i=0; i<128; i++) {
		if (note_array[i] > 0) {
			post("Note ", i, " vel ", note_array[i], '\n');
		}
	}
	
	post("Order array: \n");
	if (last_array.length > 0) {
		for (i=0; i<last_array.length; i++) {
			post("Note ", last_array[i], " vel ", note_array[last_array[i]], '\n');
		}
	}	
}

function msg_int(a) {
	var which = inlet;
	
	switch (which) {
		case 1:	
			// velocity input
			last_velo = a;
			break;
		case 0:	
			// note input
			last_note = a;
			
			if (queueNote == a) {
				queueNote = -1;
			}
			
			if (last_velo == 0) {
				del_note(last_note)
			} else {
				add_note(last_note, last_velo);
			}
			break;
	}
}

function clear() {
	if (play_note != -1) {
		sendnote(play_note, 0);
		play_note = -1;
	}
	
	note_array.splice(0, note_array.length);
	last_array.splice(0, last_array.length);
}
	

function priority(a) {
	if ((a < 0) || (a > 2)) {
		post("Invalid priority assignment");
	} else {
		curr_priority = a;
	}
}

//	Note queue handlers
//	===================

function add_note(n, v) {
	if (note_array[n] > 0) {
		note_array[n] = v;
	} else {
		note_array[n] = v;
		last_array.push(n);
	}
	
	doNoteSound();
}
add_note.local = 1;

function del_note(n) {
	var idx = last_array.indexOf(n);
	
	if (note_array[n] > 0) {
		note_array[n] = 0;
	}
	
	if (idx != -1) {
		last_array.splice(idx, 1);
	}
	
	doNoteSound();
}
del_note.local = 1;

function find_highest() {
	var i;
	
	for (i=127; i>= 0; i--) {
		if (note_array[i] > 0)	return i;
	}

	return -1;
}
find_highest.local = 1;

function find_lowest() {
	var i;
	
	for (i=0; i< 128; i++) {
		if (note_array[i] > 0)	return i;
	}

	return -1;
}
find_lowest.local = 1;

//	Note playing handlers
//	=====================

function doNoteSound() {
	var doNote = -1;
	var doVel = -1;
	
	switch (curr_priority) {
		case 0:
			// last note priority
			if (last_array.length > 0) {
				doNote = last_array[last_array.length - 1];
				doVel = note_array[doNote];
			}
			break;
		case 1:
			// high note priority
			doNote = find_highest();
			if (doNote != -1) {
				doVel = note_array[doNote];
			}
			break;
		case 2:
			// low note priority
			doNote = find_lowest();
			if (doNote != -1) {
				doVel = note_array[doNote];
			}
			break;
	}
	
	if (doNote != play_note) {
		if ((play_note != -1) || (queueNote != -1)) {
			sendnote(play_note, 0);
		}
		if (doNote != -1) {
			sendnote(doNote, doVel);
		}
	}
}
doNoteSound.local = 1;

function sendnote(n, v) {
	if (v > 0) {
		queueNote = n;
		queueVel = v;
		if (!isQueing) {
			t.schedule(QUEUEMS);
			isQueing = 1;
		}
	} else {
		if (queueNote == n) {
			queueNote = -1;
		}
		
		if (n == play_note) {
			outlet(1, v);
			outlet(0, n);
			play_note = -1;
		}
	}
}
sendnote.local = 1;

function queueplay() {
	if (queueNote != -1) {
		outlet(1, queueVel);
		outlet(0, queueNote);
		
		play_note = queueNote;
		queueNote = -1;
	}
	
	isQueing = 0;
}